/* ============================================================================
 * File Name: Dummy_01 (v0_0)
 *
 * Description:
 *   Empty pass-through component which connects two digital buses of same width.
 *   Facilitates schematic routing and resolves errors.
 *   Virtual component – does not consume resources.
 *
 * Note:
 *   bus width [1 to 32]
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using CyDesigner.Extensions.Common;
using CyDesigner.Extensions.Gde;

// The namespace is required to have the same name as the component for a customizer.
namespace Dummy_v0_0
{
    class CyCustomizer : ICyShapeCustomize_v1
    {

        public CyCustErr CustomizeShapes(ICyInstQuery_v1 instQuery, ICySymbolShapeEdit_v1 shapeEdit,
            ICyTerminalEdit_v1 termEdit)
        {
            CyCustErr err;

            // We leave the symbol as it is for symbol preview
            if (instQuery.IsPreviewCanvas)
                return CyCustErr.OK;

            // read parameters
            CyCompDevParam busWidthParam = instQuery.GetCommittedParam("bus_width");
            byte busWidth = byte.Parse(busWidthParam.Value);
            byte maxBitIndex = (byte)(busWidth - 1);

            string INPUT_TERM_NAME = "inp";
            string inTermName = termEdit.GetTermName(INPUT_TERM_NAME);
            err = termEdit.TerminalRename(inTermName, string.Format("{0}[{1}:0]", INPUT_TERM_NAME, maxBitIndex.ToString()));
            if (err.IsNotOK) return err;
        
            
            string OUTPUT_TERM_NAME = "outp";
            string outTermName = termEdit.GetTermName(OUTPUT_TERM_NAME);
            err = termEdit.TerminalRename(outTermName, string.Format("{0}[{1}:0]", OUTPUT_TERM_NAME, maxBitIndex.ToString()));
            if (err.IsNotOK) return err;
            
            
            

            return CyCustErr.OK;
        }
    }
}

//[] END OF FILE
